@@ -57,7 +57,7 @@ static GstFlowReturn gst_ffmpegauddec_handle_frame (GstAudioDecoder * decoder,
 static gboolean gst_ffmpegauddec_negotiate (GstFFMpegAudDec * ffmpegdec,
     AVCodecContext * context, AVFrame * frame, gboolean force);
 
-static void gst_ffmpegauddec_drain (GstFFMpegAudDec * ffmpegdec);
+static GstFlowReturn gst_ffmpegauddec_drain (GstFFMpegAudDec * ffmpegdec);
 
 #define GST_FFDEC_PARAMS_QDATA g_quark_from_static_string("avdec-params")
 
@@ -594,9 +594,10 @@ no_codec:
   }
 }
 
-static void
+static GstFlowReturn
 gst_ffmpegauddec_drain (GstFFMpegAudDec * ffmpegdec)
 {
+  GstFlowReturn ret = GST_FLOW_OK;
   gboolean got_any_frames = FALSE;
   gboolean got_frame;
 
@@ -604,21 +605,34 @@ gst_ffmpegauddec_drain (GstFFMpegAudDec * ffmpegdec)
     goto send_packet_failed;
 
   do {
-    GstFlowReturn ret;
-
     got_frame = gst_ffmpegauddec_frame (ffmpegdec, &ret);
     if (got_frame)
       got_any_frames = TRUE;
   } while (got_frame);
   avcodec_flush_buffers (ffmpegdec->context);
 
-  if (got_any_frames)
-    gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (ffmpegdec), NULL, 1);
+  /* FFMpeg will return AVERROR_EOF if it's internal was fully drained
+   * then we are translating it to GST_FLOW_EOS. However, because this behavior
+   * is fully internal stuff of this implementation and gstaudiodecoder
+   * baseclass doesn't convert this GST_FLOW_EOS to GST_FLOW_OK,
+   * convert this flow returned here */
+  if (ret == GST_FLOW_EOS)
+    ret = GST_FLOW_OK;
 
-  return;
+  if (got_any_frames) {
+    GstFlowReturn new_ret =
+        gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (ffmpegdec), NULL, 1);
+
+    if (ret == GST_FLOW_OK)
+      ret = new_ret;
+  }
+
+done:
+  return ret;
 
 send_packet_failed:
   GST_WARNING_OBJECT (ffmpegdec, "send packet failed, could not drain decoder");
+  goto done;
 }
 
 static void
@@ -651,8 +665,7 @@ gst_ffmpegauddec_handle_frame (GstAudioDecoder * decoder, GstBuffer * inbuf)
     goto not_negotiated;
 
   if (inbuf == NULL) {
-    gst_ffmpegauddec_drain (ffmpegdec);
-    return GST_FLOW_OK;
+    return gst_ffmpegauddec_drain (ffmpegdec);
   }
 
   inbuf = gst_buffer_ref (inbuf);
@@ -723,8 +736,18 @@ gst_ffmpegauddec_handle_frame (GstAudioDecoder * decoder, GstBuffer * inbuf)
   } while (got_frame);
 
   if (is_header || got_any_frames) {
-    ret =
+    /* Even if previous return wasn't GST_FLOW_OK, we need to call
+     * _finish_frame() since baseclass is expecting that _finish_frame()
+     * is followed by _finish_subframe()
+     */
+    GstFlowReturn new_ret =
         gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (ffmpegdec), NULL, 1);
+
+    /* Only override the flow return value if previously did have a GST_FLOW_OK.
+     * Failure to do this would result in skipping downstream issues caught in
+     * earlier steps. */
+    if (ret == GST_FLOW_OK)
+      ret = new_ret;
   }
 
 unmap:
